package cn.flyrise.pai.xlink.gree.handler;

import cn.flyrise.common.core.constant.SecurityConstants;
import cn.flyrise.common.core.domain.Reply;
import cn.flyrise.common.uc.domain.EnterpriseEntity;
import cn.flyrise.common.uc.domain.ParkModel;
import cn.flyrise.common.uc.feign.IEnterpriseService;
import cn.flyrise.common.uc.feign.IParkService;
import cn.flyrise.mybatis.feign.ITenantService;
import cn.flyrise.pai.xlink.gree.config.GreeProperties;
import cn.flyrise.pai.xlink.gree.model.GreePersonRelOrgModel;
import cn.flyrise.pai.xlink.gree.model.TagEnums;
import cn.flyrise.pai.xlink.gree.model.constants.GreeConstants;
import cn.flyrise.pai.xlink.gree.util.GreeHttpUtils;
import cn.flyrise.pai.xlink.gree.util.PageRequest;
import cn.flyrise.pai.xlink.handler.AbstractThirdPartySyncHandlerService;
import cn.flyrise.pai.xlink.handler.IThirdPartyPushStandardService;
import cn.flyrise.pai.xlink.model.constants.Constants;
import cn.flyrise.pai.xlink.model.dto.*;
import cn.flyrise.pai.xlink.model.enums.SyncTypeMapping;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.thread.ThreadUtil;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.stream.Collectors;

import static cn.hutool.core.date.DatePattern.NORM_DATETIME_PATTERN;

/**
 * 格力同步数据处理器
 *
 * @author Y.zc
 */
@Service
public class GreeSyncHandlerService extends AbstractThirdPartySyncHandlerService {

    @Resource
    private GreeProperties greeProperties;
    @Resource
    private IThirdPartyPushStandardService pushService;
    @Resource
    private GreeHttpUtils greeHttpUtils;
    @Resource
    private IEnterpriseService enterpriseService;
    @Resource
    private IParkService parkService;
    @Resource
    private ITenantService tenantService;

    private final Map<String, EnterpriseEntity> entMap = new HashMap<>(10);
    private final Map<String, ParkModel> parkMap = new HashMap<>(10);
    private final Map<String, String> tenantMap = new HashMap<>(10);

    @Override
    public String getPlatform() {
        return SyncTypeMapping.GREE.getType();
    }

    @Override
    public void syncOrg() {
        pushService.pushOrg(pullOrg(false, null), TagEnums.ORG_ALL_TAG);
    }

    @Override
    public void syncOrg(Date time) {
        pushService.pushOrg(pullOrg(true, time), TagEnums.ORG_INCR_TAG);
    }

    @Override
    public void syncUser() {
        pushService.pushOrg(pullUser(false, null), TagEnums.USER_ALL_TAG);
    }

    @Override
    public void syncUser(Date time) {
        pushService.pushOrg(pullUser(true, time), TagEnums.USER_INCR_TAG);
    }

    @Override
    public void syncRole() {
        pushService.pushOrg(pullRole(false, null), TagEnums.ROLE_ALL_TAG);
        logger.info("=====ROLE 关联等待=====");
        ThreadUtil.sleep(100);
        pushService.pushOrg(pullUserRelRole(false, null), TagEnums.USER_REL_ROLE_ALL_TAG);
    }

    @Override
    public void syncRole(Date time) {
        pushService.pushOrg(pullRole(true, time), TagEnums.ROLE_INCR_TAG);
        logger.info("=====ROLE 关联等待=====");
        ThreadUtil.sleep(100);
        pushService.pushOrg(pullUserRelRole(true, time), TagEnums.USER_REL_ROLE_INCR_TAG);
    }

    @Override
    public void syncPark() {
        pushService.pushOrg(pullPark(false, null), TagEnums.PARK_ALL_TAG);
    }

    @Override
    public void syncPark(Date time) {
        pushService.pushOrg(pullPark(true, time), TagEnums.PARK_INCR_TAG);
    }

    @Override
    public void syncSpace() {
        pushService.pushMaster(pullSpace(false, null), TagEnums.SPARK_ALL_TAG);
    }

    @Override
    public void syncSpace(Date time) {
        pushService.pushMaster(pullSpace(true, time), TagEnums.SPARK_INCR_TAG);
    }

    private PageRequest pageNew() {
        return new PageRequest();
    }

    /**
     * 拉取组织数据
     *
     * @param incr 增量
     * @param time 时间
     * @return List<OrgDTO>
     * @author Y.zc
     */
    private List<OrgDTO> pullOrg(boolean incr, Date time) {
        String url = greeProperties.getOrgDataUrl();
        List<OrgDTO> orgList = pullData(url, incr, time, OrgDTO.class);
        //处理所有的数据，先排序，按照level从低到高排序
        orgList.forEach(v -> {
            v.setStatus(1);
            v.setPlatform(getPlatform());
            v.setEntCode(greeProperties.getEntCode());
            //设置为部门
            v.setOrgType("2");
        });
        orgList.sort(Comparator.comparing(OrgDTO::getLevel));
        return orgList;
    }


    /**
     * 拉取用户数据
     *
     * @param incr 增量
     * @param time 拉取这个时间开始到现在的数据
     * @return List<UserDTO> 返回用户数据
     * @author Y.zc
     */
    private List<UserDTO> pullUser(boolean incr, Date time) {
        String url = greeProperties.getUserDataUrl();

        Future<List<UserDTO>> pullUserTask = ThreadUtil.execAsync(() -> {
            logger.info("---->> pull 人员数据");
            return pullData(url, incr, time, UserDTO.class);
        });

        //获取用户与部门的关联 获取全局的人员岗位关系
        Future<Map<String, List<String>>> pullPersonRelOrgTask = ThreadUtil.execAsync(() -> {
            logger.info("---->> pull 人员组织关系数据");
            String personRelOrgDataUrl = greeProperties.getPersonRelOrgDataUrl();
            List<GreePersonRelOrgModel> greePersonRelOrgModels = pullData(personRelOrgDataUrl, incr, time,
                    GreePersonRelOrgModel.class);

            return greePersonRelOrgModels.stream().collect(Collectors.groupingBy(GreePersonRelOrgModel::getPersonCode
                    , Collectors.mapping(GreePersonRelOrgModel::getOrgCode, Collectors.toList())));

        });
        try {
            List<UserDTO> userList = pullUserTask.get();
            Map<String, List<String>> map = pullPersonRelOrgTask.get();
            userList.forEach(v -> {
                String personCode = v.getUserId();
                v.setPlatform(getPlatform());
                v.setOrgIds(map.get(personCode));
                v.setEntCode(greeProperties.getEntCode());
                //设置工作状态以及用户状态
                String status = v.getStatus();
                String jobStatus = v.getJobStatus();
                if (GreeConstants.JOB_WORK.equals(jobStatus)) {
                    v.setJobStatus(Constants.Staff.USER_STATE_ENABLE);
                } else if (GreeConstants.JOB_LEAVE.equals(jobStatus)) {
                    v.setJobStatus(Constants.Staff.USER_STATE_LEAVE);
                }
                if (GreeConstants.USER_STATE_DISABLE.equals(status)) {
                    v.setStatus(Constants.User.USER_STATE_DISABLE);
                } else if (GreeConstants.USER_STATE_ENABLE.equals(status) || GreeConstants.USER_STATE_NOT_ACTIVATED.equals(status)) {
                    v.setStatus(Constants.User.USER_STATE_ENABLE);
                }
            });
            return userList;
        } catch (InterruptedException e) { //异常
            logger.error("执行同步用户出错--->>", e);
            Thread.currentThread().interrupt();
        } catch (ExecutionException e) {
            logger.error("执行同步用户出错--->>", e);
        } finally {
            pullUserTask.cancel(true);
            pullPersonRelOrgTask.cancel(true);
        }
        return new ArrayList<>();
    }

    /**
     * 拉取角色数据
     *
     * @param incr 增量
     * @param time 拉取这个时间开始到现在的数据
     * @return List<RoleDTO> 返回用户数据
     * @author Y.zc
     */
    private List<RoleDTO> pullRole(boolean incr, Date time) {
        String url = greeProperties.getRoleDataUrl();
        logger.info("---->> pull 角色数据");
        List<RoleDTO> roleList = pullData(url, incr, time, RoleDTO.class);
        roleList.forEach(v -> {
            v.setEntCode(greeProperties.getEntCode());
            v.setStatus("1");
        });
        return roleList;
    }

    /**
     * 拉取人员角色关联数据
     *
     * @param incr 增量
     * @param time 拉取这个时间开始到现在的数据
     * @return List<UserRelRoleDTO> 返回用户数据
     * @author Y.zc
     */
    private List<UserRelRoleDTO> pullUserRelRole(boolean incr, Date time) {
        String personRelRoleDataUrl = greeProperties.getPersonRelRoleDataUrl();
        String entCode = greeProperties.getEntCode();
        logger.info("---->> pull 人员角色关联数据");
        List<UserRelRoleDTO> userRelRoleList = pullData(personRelRoleDataUrl, incr, time, UserRelRoleDTO.class);
        userRelRoleList.forEach(v -> v.setEntCode(entCode));
        return userRelRoleList;
    }

    /**
     * 拉取空间数据
     *
     * @param incr 增量
     * @param time 拉取这个时间开始到现在的数据
     * @return List<SpaceDTO> 返回空间数据
     */
    private List<ParkDTO> pullPark(boolean incr, Date time) {
        String spaceResourceUrl = greeProperties.getSpaceResourceUrl();
        String entCode = greeProperties.getEntCode();
        logger.info("---->> pull 园区数据");
        Map<String, String> custom = new HashMap<>();
        custom.put("type", "0");
        List<ParkDTO> spaceList = pullDataCustom(spaceResourceUrl, incr, time, custom, ParkDTO.class);
        spaceList.forEach(v -> v.setEntCode(entCode));
        return spaceList;
    }

    /**
     * 拉取空间数据
     *
     * @param incr 增量
     * @param time 拉取这个时间开始到现在的数据
     * @return List<SpaceDTO> 返回空间数据
     */
    private List<SpaceDTO> pullSpace(boolean incr, Date time) {
        String spaceResourceUrl = greeProperties.getSpaceResourceUrl();
        String entCode = greeProperties.getEntCode();
        logger.info("---->> pull 空间数据");
        List<SpaceDTO> spaceList = pullData(spaceResourceUrl, incr, time, SpaceDTO.class);
        EnterpriseEntity enterprise = getEnt(entCode);
        spaceList.forEach(v -> {
            String projectCode = v.getProjectCode();
            ParkModel parkModel = getPark(projectCode);

            if (parkModel != null) {
                v.setPaiParkId(parkModel.getParkId());
                v.setPaiParkName(parkModel.getParkName());
            } else {
                v.setPaiParkId("-1");
                v.setPaiParkName("未找到园区");
            }
            if (enterprise != null) {
                v.setPaiEntId(enterprise.getEntId());
                v.setPaiEntName(enterprise.getEntName());
                String tenantId = getTenantId(enterprise.getEntId());
                if (CharSequenceUtil.isNotBlank(tenantId)) {
                    v.setPaiTenantId(tenantId);
                }
            } else {
                v.setPaiEntId("-1");
                v.setPaiEntName("未找到企业信息");
            }
            v.setEntCode(entCode);
        });
        return spaceList;
    }

    /**
     * 拉取数据
     *
     * @param url  请求地址
     * @param incr 增量
     * @param t    泛型
     * @return List<T>
     */
    private <T> List<T> pullData(String url, boolean incr, Date time, Class<T> t) {
        return pullDataCustom(url, incr, time, null, t);
    }

    /**
     * 拉取数据
     *
     * @param url    请求地址
     * @param incr   增量
     * @param custom custom
     * @param t      泛型
     * @return List<T>
     */
    private <T> List<T> pullDataCustom(String url, boolean incr, Date time, Map<String, String> custom, Class<T> t) {
        PageRequest pageRequest = pageNew();
        if (incr) {
            pageRequest.add("time", DateUtil.format(time, NORM_DATETIME_PATTERN));
        }
        if (custom != null && !custom.isEmpty()) {
            custom.forEach(pageRequest::add);
        }
        return greeHttpUtils.greePageAllPost(url, pageRequest, t);
    }

    private EnterpriseEntity getEnt(String code) {
        if (entMap.containsKey(code)) {
            return entMap.get(code);
        }
        Reply<EnterpriseEntity> reply = enterpriseService.findByCode(code, SecurityConstants.FROM_IN);
        if (reply.isSuccess() && reply.getData() != null) {
            EnterpriseEntity enterpriseEntity = reply.getData();
            entMap.put(code, enterpriseEntity);
            return enterpriseEntity;
        }
        return null;
    }

    private String getTenantId(String entId) {
        if (tenantMap.containsKey(entId)) {
            return tenantMap.get(entId);
        }
        Reply<String> reply = tenantService.getGroupIdByEntId(entId, SecurityConstants.FROM_IN);
        if (reply.isSuccess() && reply.getData() != null) {
            String tenantId = reply.getData();
            tenantMap.put(entId, tenantId);
            return tenantId;
        }
        return null;
    }

    private ParkModel getPark(String code) {
        if (parkMap.containsKey(code)) {
            return parkMap.get(code);
        }
        Reply<ParkModel> reply = parkService.findByCode(code, SecurityConstants.FROM_IN);
        if (reply.isSuccess() && reply.getData() != null) {
            ParkModel parkModel = reply.getData();
            parkMap.put(code, parkModel);
            return parkModel;
        }
        return null;
    }

}
