package cn.flyrise.pai.xlink.gree.sso;

import cn.flyrise.common.security.domain.LoginUser;
import cn.flyrise.common.security.utils.SecurityUtils;
import cn.flyrise.pai.xlink.gree.config.GreeProperties;
import cn.flyrise.pai.xlink.gree.util.GreeHttpUtils;
import cn.flyrise.pai.xlink.handler.ISsoHandlerService;
import cn.flyrise.pai.xlink.model.po.LinkSsoTokenPO;
import cn.flyrise.pai.xlink.service.ILinkSsoTokenService;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.SecureUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;

import static cn.flyrise.pai.xlink.gree.model.constants.GreeConstants.*;

/**
 * 格力单点登录
 *
 * @author Y.zc
 */
@Component("greeSsoHandlerService")
public class GreeSsoHandlerServiceImpl implements ISsoHandlerService {
    protected Logger logger = LoggerFactory.getLogger(getClass());

    @Resource
    private GreeProperties greeProperties;
    @Resource
    private GreeHttpUtils greeHttpUtils;
    @Resource
    private ILinkSsoTokenService ssoTokenService;

    @Override
    public String getUserCode(String code, String state) {
        String appId = greeProperties.getAppId();
        String appSecret = greeProperties.getAppSecret();

        String tokenUrlFormat = "{tokenUrl}?response_type=code" +
                "&client_id={clientId}&redirect_uri={redirectUri}&scope=read&code={code}" +
                "&client_secret={clientSecret}&grant_type=authorization_code";
        Map<String, String> params = new HashMap<>(10);
        params.put("tokenUrl", greeProperties.getTokenUrl());
        params.put("clientId", appId);
        params.put("clientSecret", appSecret);
        params.put("redirectUri", greeProperties.getRedirectUri());
        params.put("code", code);
        String tokenUrl = StrUtil.format(tokenUrlFormat, params, true);
        String body = greeHttpUtils.post(tokenUrl, null);
        //获取用户信息
        JSONObject tokenJson = JSON.parseObject(body);
        if (tokenJson.containsKey(ACCESS_TOKEN)) {
            String token = tokenJson.getString(ACCESS_TOKEN);
            String userBody = greeHttpUtils.authPost(greeProperties.getUserUrl(), null, TOKEN_TYPE + token);
            JSONObject userJson = JSON.parseObject(userBody);
            if (userJson.containsKey(PERSON_CODE)) {
                //设置SSO_TOKEN
                String personCode = userJson.getString(PERSON_CODE);
                String account = userJson.getString(ACCOUNT);
                LinkSsoTokenPO ssoToken = ssoTokenService.findByPlatformAndUniId(PLATFORM, account);
                if (ssoToken != null) {
                    //注销账号
                    logout(ssoToken.getSsoToken());
                    ssoTokenService.delByPlatformAndUniId(PLATFORM, account);
                }
                ssoTokenService.addToken(PLATFORM, account, token);
                return SecureUtil.md5(personCode);
            }
        } else {
            logger.error("Gree Get Token ERR:{}", body);
        }
        return null;
    }

    @Override
    public String logout() {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        if (loginUser != null) {
            String account = loginUser.getUsername();
            LinkSsoTokenPO ssoToken = ssoTokenService.findByPlatformAndUniId(PLATFORM, account);
            if (ssoToken != null) {
                ssoTokenService.delByPlatformAndUniId(PLATFORM, account);
                return logout(ssoToken.getSsoToken());
            }
        }
        return logout(null);
    }

    private String logout(String token) {
        String logoutUrl = greeProperties.getLogoutUrl();
        String body = greeHttpUtils.authPost(logoutUrl, null, TOKEN_TYPE + token);
        JSONObject jsonObject = JSON.parseObject(body);
        if (jsonObject.containsKey(RESPONSE_CODE)) {
            JSONObject data = jsonObject.getJSONObject(RESPONSE_DATA);
            return data.getString("portalWebUrl");
        }
        return null;
    }
}
