package cn.flyrise.pai.xlink.gree.util;

import cn.flyrise.pai.xlink.gree.GreeConstants;
import cn.flyrise.pai.xlink.gree.config.GreeProperties;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import org.apache.http.entity.ContentType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 格力Http请求
 *
 * @author Y.zc
 */
@Component
public class GreeHttpUtils {

    static Logger logger = LoggerFactory.getLogger(GreeHttpUtils.class);

    @Resource
    private GreeProperties greeProperties;
    @Resource
    private GreePageResponseResolver pageResponseResolver;

    /**
     * Post 请求 - static
     *
     * @param appId     appId
     * @param appSecret appSecret
     * @param url       请求地址
     * @param body      请求参数
     * @return String
     * @author Y.zc
     */
    public static String post(String appId, String appSecret, String url, String body, String token) {
        logger.info("gree req url：{},body:{}", url, body);
        long time = System.currentTimeMillis();
        String sign = GreeSignUtils.sign(appId, appSecret, time, body);
        HttpRequest httpRequest = HttpUtil.createPost(url)
                .header(GreeConstants.Sign.HEAD_APP_ID, appId)
                .header(GreeConstants.Sign.HEAD_APP_SIGN, sign)
                .header(GreeConstants.Sign.HEAD_APP_TIME, time + "");
        if (body != null) {
            httpRequest.body(body, ContentType.APPLICATION_JSON.getMimeType());
        }
        if (token != null) {
            httpRequest.header(GreeConstants.Auth.HEAD_BASIC, token);
            httpRequest.header(GreeConstants.Auth.HEAD_ACCESS_TOKEN, token);
        }
        HttpResponse response = httpRequest.execute();
        String responseBody = response.body();
        logger.info("gree resp body：{}", responseBody);
        return responseBody;
    }

    public String authPost(String url, String body, String token) {
        String appId = greeProperties.getAppId();
        String appSecret = greeProperties.getAppSecret();
        String serverUri = greeProperties.getServerUri();
        return post(appId, appSecret, serverUri + url, body, token);
    }

    public static String post(String appId, String appSecret, String url, String body) {
        return post(appId, appSecret, url, body, null);
    }

    public String post(String url, String body) {
        String appId = greeProperties.getAppId();
        String appSecret = greeProperties.getAppSecret();
        String serverUri = greeProperties.getServerUri();
        return post(appId, appSecret, serverUri + url, body);
    }

    public <T> List<T> greePageAllPost(String url, PageRequest request, Class<T> t) {
        String appId = greeProperties.getAppId();
        String appSecret = greeProperties.getAppSecret();
        String serverUri = greeProperties.getServerUri();
        return pageAllPost(appId, appSecret, serverUri + url, request, t);
    }


    public <T> List<T> pageAllPost(String appId, String appSecret, String url, PageRequest request, Class<T> t) {
        //初始化分页参数,从1开始
        Map<String, Object> reqMap = new HashMap<>(2);
        reqMap.put(request.getPageNoColumn(), request.getPageNo());
        reqMap.put(request.getPageSizeColumn(), request.getPageSize());
        if (!request.getReqMap().isEmpty()) {
            reqMap.putAll(request.getReqMap());
        }
        List<T> list = new ArrayList<>();

        while (true) {
            String body = JSON.toJSONString(reqMap);
            long time = System.currentTimeMillis();
            String sign = GreeSignUtils.sign(appId, appSecret, time, body);
            HttpRequest httpRequest = HttpUtil.createPost(url)
                    .body(JSON.toJSONString(reqMap), ContentType.APPLICATION_JSON.getMimeType())
                    .header(GreeConstants.Sign.HEAD_APP_ID, appId)
                    .header(GreeConstants.Sign.HEAD_APP_SIGN, sign)
                    .header(GreeConstants.Sign.HEAD_APP_TIME, time + "");
            HttpResponse response = httpRequest.execute();
            String respBody = response.body();
            logger.info("获取第{}页,body:{}", request.getPageNo(), respBody);
            PageResponse<T> pageResp = pageResponseResolver.resolver(respBody, t);
            list.addAll(pageResp.getRows());
            if (pageResp.getTotalPage() > request.getPageNo()) {
                request.setPageNo(request.getPageNo() + 1);
                reqMap.put(request.getPageNoColumn(), request.getPageNo());
            } else {
                break;
            }
        }
        return list;
    }


}
