package cn.flyrise.pai.xlink.gree.util;

import cn.flyrise.pai.xlink.annotation.XLink;
import cn.flyrise.pai.xlink.service.ILinkOrgDataParseService;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.date.TimeInterval;
import cn.hutool.core.text.CharSequenceUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.JSONPath;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * 格力分页请求解析器
 *
 * @author Y.zc
 */
@Component
public class GreePageResponseResolver {

    private final Logger logger = LoggerFactory.getLogger(GreePageResponseResolver.class);

    @Resource
    private ILinkOrgDataParseService linkOrgDataParseService;


    private static final PageResponseColumn PAGE_RESPONSE_COLUMN = new PageResponseColumn();

    public <T> PageResponse<T> resolver(String respBody, Class<T> t) {

        JSONObject respJson = JSON.parseObject(respBody);
        if (PAGE_RESPONSE_COLUMN.getSuccessCode().equals(respJson.getString(PAGE_RESPONSE_COLUMN.getStatusCodeColumn()))) {
            JSONObject data = respJson;
            if (CharSequenceUtil.isNotBlank(PAGE_RESPONSE_COLUMN.getDataColumn())) {
                data = respJson.getJSONObject(PAGE_RESPONSE_COLUMN.getDataColumn());
            }
            Integer pageIndex = data.getInteger(PAGE_RESPONSE_COLUMN.getPageIndexColumn());
            Integer pageSize = data.getInteger(PAGE_RESPONSE_COLUMN.getPageSizeColumn());
            Integer totalPage = data.getInteger(PAGE_RESPONSE_COLUMN.getTotalPageColumn());
            Integer totalCount = data.getInteger(PAGE_RESPONSE_COLUMN.getTotalCountColumn());
            //可以使用反序列化调整JSON的key
            TimeInterval time = DateUtil.timer();
            if (t.isAnnotationPresent(XLink.class)) {
                XLink xLink = t.getAnnotation(XLink.class);
                String value = xLink.value();
                Map<String, String> parseMap = linkOrgDataParseService.getParseByType(value);
                JSONArray dataArray = data.getJSONArray(PAGE_RESPONSE_COLUMN.getRowsColumn());
                //test1,使用JSONPath
                for (int i = 0; i < dataArray.size(); i++) {
                    JSONObject oneDate = dataArray.getJSONObject(i);
                    String jsonStr = oneDate.toString();
                    parseMap.forEach((k, v) -> {
                        if (JSONPath.contains(oneDate, k)) {
                            Object o = JSONPath.extract(jsonStr, k);
                            oneDate.put(v, o);
                        }
                    });
                }
                logger.info("parse json time:{}ms", time.intervalRestart());
                List<T> rows = dataArray.toJavaList(t);
                logger.info("parse entity time:{}ms", time.intervalRestart());
                return new PageResponse<>(pageIndex, pageSize, totalPage, totalCount, rows);
            } else {
//                List<T> rows = data.getList(PAGE_RESPONSE_COLUMN.getRowsColumn(), t);
                List<T> rows = JSON.parseArray(data.getString(PAGE_RESPONSE_COLUMN.getRowsColumn()), t);
                return new PageResponse<>(pageIndex, pageSize, totalPage, totalCount, rows);
            }
        } else {
            logger.warn("错误信息：{}", respBody);
            return new PageResponse<>();
        }
    }

}
