package cn.flyrise.pai.xlink.handler;

import java.util.Date;

/**
 * 统一同步Handler接口
 *
 * @author Y.zc
 */
public interface IThirdPartySyncHandlerService {

    /**
     * 获取同步类型
     *
     * @return String
     * @author Y
     */
    String getPlatform();

    /**
     * 全量同步
     *
     * @author Y.zc
     */
    default void allSync() {
        syncOrg();
        syncUser();
        syncPost();
        syncRole();
        syncPark();
        syncSpace();
        syncOther();
    }

    /**
     * 统一同步方法
     *
     * @param date 上次同步的时间
     * @author Y.zc
     */
    default void incrSync(Date date) {
        syncOrg(date);
        syncUser(date);
        syncPost(date);
        syncRole(date);
        syncPark(date);
        syncSpace(date);
        syncOther(date);
    }

    /**
     * 同步组织机构
     *
     * @author Y.zc
     */
    void syncOrg();

    /**
     * 增量同步组织机构
     *
     * @param date 上次同步的时间
     * @author Y.zc
     */
    void syncOrg(Date date);

    /**
     * 全量同步员工/用户
     *
     * @author Y.zc
     */
    void syncUser();

    /**
     * 增量同步员工/用户
     *
     * @param date 上次同步的时间
     * @author Y.zc
     */
    void syncUser(Date date);

    /**
     * 全量同步岗位
     *
     * @author Y.zc
     */
    void syncPost();

    /**
     * 增量同步岗位
     *
     * @param date 上次同步的时间
     * @author Y.zc
     */
    void syncPost(Date date);

    /**
     * 全量同步角色
     *
     * @author Y.zc
     */
    void syncRole();

    /**
     * 增量同步角色
     *
     * @param date 上次同步的时间
     * @author Y.zc
     */
    void syncRole(Date date);

    /**
     * 全量同步园区数据
     *
     * @author Y.zc
     */
    void syncPark();

    /**
     * 增量同步园区数据
     *
     * @param date 上次同步的时间
     * @author Y.zc
     */
    void syncPark(Date date);

    /**
     * 全量同步空间数据
     *
     * @author Y.zc
     */
    void syncSpace();

    /**
     * 增量同步空间数据
     *
     * @param date 上次同步的时间
     * @author Y.zc
     */
    void syncSpace(Date date);

    /**
     * 全量同步其他业务数据
     *
     * @author Y.zc
     */
    void syncOther();

    /**
     * 增量同步其他业务数据
     *
     * @param date 上次同步的时间
     * @author Y.zc
     */
    void syncOther(Date date);

}
