package cn.flyrise.pai.xlink.handler.impl;

import cn.flyrise.common.core.domain.Reply;
import cn.flyrise.common.security.feign.IAuthCoreService;
import cn.flyrise.pai.xlink.config.XlinkProperties;
import cn.flyrise.pai.xlink.exception.XLinkBizException;
import cn.flyrise.pai.xlink.gree.config.GreeProperties;
import cn.flyrise.pai.xlink.handler.ISsoHandlerService;
import cn.flyrise.pai.xlink.model.enums.SsoTypeMapping;
import cn.flyrise.pai.xlink.model.enums.SyncTypeMapping;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.log.StaticLog;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.RequestParam;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Map;
import java.util.Objects;

import static cn.flyrise.pai.xlink.exception.BusinessErrors.*;

/**
 * 单点登录处理器
 *
 * @author Y.zc
 */
@Service
public class SsoHandler {

    protected Logger logger = LoggerFactory.getLogger(getClass());
    @Resource
    private Map<String, ISsoHandlerService> ssoHandlerServiceMap;
    @Resource
    private IAuthCoreService authCoreService;
    @Resource
    private XlinkProperties xlinkProperties;

    public String getOpenId(String platform, String code, String state) {
        ISsoHandlerService ssoHandlerService = getHandler(platform);
        return ssoHandlerService.getUserCode(code, state);
    }

    public void sso(String platform, String code, String state, HttpServletResponse response) throws IOException {
        String openId = getOpenId(platform, code, state);
        if (CharSequenceUtil.isBlank(openId)) {
            throw new XLinkBizException(ERR_4000);
        }
        //第三方登录进凌云
        String paiState = getPaiState(platform);

        String url = CharSequenceUtil.format("{}/auth/?openId={}&type={}&state={}", xlinkProperties.getSsoResponseUri(), openId,
                "workbench", paiState);
        //模块跳转
        StaticLog.info("===>> RedirectUrl:{}", url);
        response.sendRedirect(url);
    }

    public void logout(String platform, HttpServletResponse response) {
        ISsoHandlerService ssoHandlerService = getHandler(platform);
        String logoutUrl = ssoHandlerService.logout();
        try {
            if (CharSequenceUtil.isNotBlank(logoutUrl)) {
                response.sendRedirect(logoutUrl);
            } else {
                response.sendRedirect("https://portal-sso-sit.spaceplat.com/ubmp-workbench-view");
            }
        } catch (Exception e) {
            logger.error("格力退出失败：", e);
            throw new XLinkBizException(ERR_5002);
        }

    }


    public String getPaiState(String platform) {
        Reply<String> reply = authCoreService.getAuthorizeState(platform, "workbench");
        return reply.getData();
    }

    private ISsoHandlerService getHandler(String platform) {
        logger.info("获取单点登录处理器=====>>>>");
        SsoTypeMapping ssoTypeMapping = SsoTypeMapping.of(platform);
        if (Objects.nonNull(ssoTypeMapping)) {
            return ssoHandlerServiceMap.get(ssoTypeMapping.getHandlerName());
        } else {
            logger.warn("找不到单点登录的平台:{}", platform);
            throw new XLinkBizException(ERR_5000, platform);
        }
    }
}
