package cn.flyrise.pai.xlink.handler.impl;

import cn.flyrise.pai.xlink.config.XlinkProperties;
import cn.flyrise.pai.xlink.exception.XLinkBizException;
import cn.flyrise.pai.xlink.handler.IThirdPartySyncHandlerService;
import cn.flyrise.pai.xlink.model.enums.SyncTypeMapping;
import cn.flyrise.pai.xlink.model.po.LinkOrgSyncLogPO;
import cn.flyrise.pai.xlink.service.ILinkOrgSyncInfoService;
import cn.flyrise.pai.xlink.service.ILinkOrgSyncLogService;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.date.TimeInterval;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.Map;
import java.util.Objects;

import static cn.flyrise.pai.xlink.exception.BusinessErrors.ERR_5000;
import static cn.flyrise.pai.xlink.exception.BusinessErrors.ERR_5001;

/**
 * 第三方同步统一执行
 *
 * @author Y.zc
 */
@Service
public class ThirdPartySyncHandler {

    protected Logger logger = LoggerFactory.getLogger(getClass());

    @Resource
    private XlinkProperties xlinkProperties;
    @Resource
    private Map<String, IThirdPartySyncHandlerService> thirdPartySyncHandlerServiceMap;
    @Resource
    private ILinkOrgSyncInfoService linkOrgSyncInfoService;
    @Resource
    private ILinkOrgSyncLogService linkOrgSyncLogService;


    public void allSync() {
        sync(false);
    }

    public void incrSync() {
        sync(true);
    }


    public void allMaster() {
        IThirdPartySyncHandlerService handlerService = getHandler();
        handlerService.syncSpace();
    }

    private synchronized void sync(boolean incr) {
        String platform = xlinkProperties.getPlatform();
        Date now = DateUtil.date();
        //获取上一次同步成功的时间
        Date time = linkOrgSyncInfoService.getLastSyncTime();
        TimeInterval interval = DateUtil.timer();
        //日志记录
        boolean success = false;
        String title = "全量同步";
        if (incr) {
            title = "增量同步";
        }
        LinkOrgSyncLogPO orgSyncLog = linkOrgSyncLogService.init(title, time);
        IThirdPartySyncHandlerService handlerService = getHandler();
        try {
            if (incr) {
                handlerService.incrSync(time);
            } else {
                handlerService.allSync();
            }
            success = true;
            orgSyncLog.setContent("OK");
            linkOrgSyncInfoService.saveInfo(now);
        } catch (Exception e) {
            orgSyncLog.setContent("ERR:" + e.getMessage());
            logger.error("同步Platform:[{}]失败", platform, e);
        }
        orgSyncLog.setSuccess(success ? 1 : 0);
        orgSyncLog.setExecutionTime(interval.interval());
        linkOrgSyncLogService.update(orgSyncLog);
    }

    private IThirdPartySyncHandlerService getHandler() {
        if (xlinkProperties.isEnable()) {
            String platform = xlinkProperties.getPlatform();
            logger.info("执行同步=====>>>>");
            SyncTypeMapping syncTypeMapping = SyncTypeMapping.of(platform);
            if (Objects.nonNull(syncTypeMapping)) {
                return thirdPartySyncHandlerServiceMap.get(syncTypeMapping.getHandlerName());
            } else {
                logger.warn("找不到同步的平台:{}", platform);
                throw new XLinkBizException(ERR_5000, platform);
            }
        } else {
            logger.warn("未开启同步，请确认参数是否正确");
            throw new XLinkBizException(ERR_5001);
        }
    }

}
