package cn.flyrise.pai.xlink.handler.impl;

import cn.flyrise.mq.core.model.PaiMqMessageBean;
import cn.flyrise.mq.core.service.PaiMqTemplate;
import cn.flyrise.pai.xlink.config.XlinkProperties;
import cn.flyrise.pai.xlink.gree.model.TagEnums;
import cn.flyrise.pai.xlink.handler.IThirdPartyPushStandardService;
import cn.flyrise.pai.xlink.model.constants.Constants;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ReflectUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * <p>
 * 标准实现
 * 组织机构、人员、岗位、角色
 * </p>
 *
 * @author Y.zc
 */
@Service
public class ThirdPartySyncStandardServiceImpl implements IThirdPartyPushStandardService {

    private static final Logger logger = LoggerFactory.getLogger(ThirdPartySyncStandardServiceImpl.class);

    @Resource
    private XlinkProperties xlinkProperties;
    @Resource
    private PaiMqTemplate<Object> paiMqTemplate;

    @Override
    public <T> void pushOrg(List<T> list, TagEnums tagEnums) {
        String dateNow = DateUtil.now();
        String topic = xlinkProperties.getOrgTopic();
        pushData(topic, list, tagEnums.getCode());
        logger.info("{} 推送{},total:{}条", dateNow, tagEnums.getDesc(), list.size());
    }

    @Override
    public <T> void pushOrg(List<T> list, String tag, String desc) {
        String dateNow = DateUtil.now();
        String topic = xlinkProperties.getOrgTopic();
        pushData(topic, list, tag);
        logger.info("{} 推送{},total:{}条", dateNow, desc, list.size());
    }

    @Override
    public <T> void pushMaster(List<T> list, String tag, String desc) {
        String dateNow = DateUtil.now();
        String topic = xlinkProperties.getMasterTopic();
        pushData(topic, list, tag);
        logger.info("{} 推送{},total:{}条", dateNow, desc, list.size());
    }

    @Override
    public <T> void pushMaster(List<T> list, TagEnums tagEnums) {
        String dateNow = DateUtil.now();
        String topic = xlinkProperties.getMasterTopic();
        pushData(topic, list, tagEnums.getCode());
        logger.info("{} 推送{},total:{}条", dateNow, tagEnums.getDesc(), list.size());
    }


    private <T> void pushData(String topic, List<T> list, String tag) {
        if (!list.isEmpty()) {
            //推送消息-->> pai-console
            for (T t : list) {
                logger.info("推送数据{}-->>{}", tag, t);
                String key = ReflectUtil.invoke(t, "getId");
                PaiMqMessageBean<Object> messageBean = new PaiMqMessageBean.Builder<>()
                        .tag(tag)
                        .status(Constants.Status.SYNC)
                        .key(key)
                        .data(t)
                        .build();
                String msgId = paiMqTemplate.send(topic, messageBean);
                logger.debug("msgId:{}", msgId);
            }
        }
    }
}
