package cn.flyrise.pai.xlink.service.impl;


import cn.flyrise.mybatis.base.BaseServiceImpl;
import cn.flyrise.pai.xlink.dao.LinkOrgDataParseMapper;
import cn.flyrise.pai.xlink.model.po.LinkOrgDataParsePO;
import cn.flyrise.pai.xlink.model.vo.LinkOrgDataParseVO;
import cn.flyrise.pai.xlink.service.ILinkOrgDataParseService;
import cn.hutool.log.StaticLog;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.PostConstruct;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 字段映射服务实现类
 *
 * @author pai
 * @date 2023-02-24
 */
@Service("linkOrgDataParse")
@Transactional(rollbackFor = RuntimeException.class)
public class LinkOrgDataParseServiceImpl extends BaseServiceImpl<LinkOrgDataParseMapper, LinkOrgDataParsePO, LinkOrgDataParseVO> implements ILinkOrgDataParseService {

    private final Map<String, Map<String, String>> parseMap = new HashMap<>();

    @PostConstruct
    public void init() {
        LambdaQueryWrapper<LinkOrgDataParsePO> qw = Wrappers.<LinkOrgDataParsePO>lambdaQuery()
                .isNotNull(LinkOrgDataParsePO::getOriginalKey)
                .ne(LinkOrgDataParsePO::getOriginalKey, "");
        List<LinkOrgDataParsePO> orgDataParseList = baseMapper.selectList(qw);
        parseMap.putAll(orgDataParseList.stream().collect(Collectors.groupingBy(LinkOrgDataParsePO::getParseType,
                Collectors.toMap(LinkOrgDataParsePO::getOriginalKey, LinkOrgDataParsePO::getParseKey))));

        StaticLog.info("初始化转换参数:{}", parseMap);
    }

    @Override
    public Map<String, String> getParseByType(String type) {

        return parseMap.get(type);
    }
}
