package cn.flyrise.pai.xlink.service.impl;


import cn.flyrise.pai.xlink.config.XlinkProperties;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import cn.flyrise.common.core.utils.IdUtils;
import cn.flyrise.pai.xlink.model.po.LinkOrgSyncInfoPO;

import javax.annotation.Resource;

import cn.flyrise.common.core.exception.CommonException;
import cn.flyrise.pai.xlink.service.ILinkOrgSyncInfoService;
import cn.flyrise.pai.xlink.dao.LinkOrgSyncInfoMapper;
import cn.flyrise.mybatis.base.BaseServiceImpl;
import cn.flyrise.pai.xlink.model.vo.LinkOrgSyncInfoVO;
import cn.hutool.core.util.StrUtil;
import cn.flyrise.common.core.utils.ConvertUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;

/**
 * 同步的配置信息服务实现类
 *
 * @author pai
 * @date 2023-03-02
 */
@Service("linkOrgSyncInfo")
@Transactional(rollbackFor = RuntimeException.class)
public class LinkOrgSyncInfoServiceImpl extends BaseServiceImpl<LinkOrgSyncInfoMapper, LinkOrgSyncInfoPO, LinkOrgSyncInfoVO> implements ILinkOrgSyncInfoService {

    @Resource
    private XlinkProperties xlinkProperties;

    @Override
    public void saveInfo(Date startTime) {
        String platform = xlinkProperties.getPlatform();
        LinkOrgSyncInfoPO orgSyncInfo = findByPlatform(platform);
        if (orgSyncInfo == null) {
            orgSyncInfo = new LinkOrgSyncInfoPO();
            orgSyncInfo.setId(IdUtils.snowFlakeId());
            orgSyncInfo.setPlatform(platform);
            orgSyncInfo.setLastSyncTime(startTime);
            insert(orgSyncInfo);
        } else {
            orgSyncInfo.setLastSyncTime(startTime);
            update(orgSyncInfo);
        }
    }

    @Override
    public Date getLastSyncTime() {
        String platform = xlinkProperties.getPlatform();
        LinkOrgSyncInfoPO orgSyncInfo = findByPlatform(platform);
        if (orgSyncInfo != null) {
            return orgSyncInfo.getLastSyncTime();
        }
        return null;
    }

    private LinkOrgSyncInfoPO findByPlatform(String platform) {
        LambdaQueryWrapper<LinkOrgSyncInfoPO> qw = Wrappers.<LinkOrgSyncInfoPO>lambdaQuery()
                .eq(LinkOrgSyncInfoPO::getPlatform, platform);
        return selectOnePO(qw);
    }
}
