package cn.flyrise.pai.xlink.service.impl;


import cn.flyrise.common.core.utils.IdUtils;
import cn.flyrise.mybatis.base.BaseServiceImpl;
import cn.flyrise.pai.xlink.dao.LinkSsoTokenMapper;
import cn.flyrise.pai.xlink.model.po.LinkSsoTokenPO;
import cn.flyrise.pai.xlink.model.vo.LinkSsoTokenVO;
import cn.flyrise.pai.xlink.service.ILinkSsoTokenService;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * 单点token服务实现类
 *
 * @author pai
 * @date 2023-02-24
 */
@Service("linkSsoToken")
@Transactional(rollbackFor = RuntimeException.class)
public class LinkSsoTokenServiceImpl extends BaseServiceImpl<LinkSsoTokenMapper, LinkSsoTokenPO, LinkSsoTokenVO> implements ILinkSsoTokenService {

    @Override
    public LinkSsoTokenPO findByPlatformAndUniId(String platform, String uniId) {

        return selectOnePO(platformAndUniIdQw(platform,uniId));
    }

    @Override
    public void addToken(String platform, String uniId, String token) {
        LinkSsoTokenPO ssoToken = new LinkSsoTokenPO();
        ssoToken.setSsoId(IdUtils.snowFlakeId());
        ssoToken.setSsoPlatform(platform);
        ssoToken.setSsoUniId(uniId);
        ssoToken.setSsoToken(token);
        ssoToken.setCreateTime(DateUtil.date());
        insert(ssoToken);
    }

    @Override
    public void delByPlatformAndUniId(String platform, String uniId) {
        remove(platformAndUniIdQw(platform,uniId));
    }

    private LambdaQueryWrapper<LinkSsoTokenPO> platformAndUniIdQw(String platform, String uniId) {
        return Wrappers.<LinkSsoTokenPO>lambdaQuery()
                .eq(LinkSsoTokenPO::getSsoPlatform, platform)
                .eq(LinkSsoTokenPO::getSsoUniId, uniId);
    }
}
