package cn.flyrise.pai.xlink.xxljob;

import cn.flyrise.job.annotation.XxlJobTask;
import cn.flyrise.pai.xlink.config.XlinkProperties;
import cn.flyrise.pai.xlink.handler.impl.ThirdPartySyncHandler;
import cn.flyrise.pai.xlink.model.po.LinkOrgSyncLogPO;
import cn.flyrise.pai.xlink.service.ILinkOrgSyncInfoService;
import cn.flyrise.pai.xlink.service.ILinkOrgSyncLogService;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.date.TimeInterval;
import cn.hutool.log.StaticLog;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import com.xxl.job.core.log.XxlJobLogger;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Date;

/**
 * 同步数据
 *
 * @author Y.zc
 */
@Component
public class SyncData {
    private final Logger logger = LoggerFactory.getLogger(getClass());

    @Resource
    private XlinkProperties xlinkProperties;
    @Resource
    private ThirdPartySyncHandler thirdPartySyncHandler;
    @Resource
    private ILinkOrgSyncLogService linkOrgSyncLogService;

    @XxlJob("xlinkSyncAllData")
    @XxlJobTask(desc = "${xxl.xlink.syncAllOrgData.desc:全量同步数据，每天凌晨2点执行}",
            cron = "${xxl.xlink.syncAllOrgData.cron:0 30 2 * * ?}",
            author = "pai-x-link")
    public ReturnT<String> syncAllOrg(String param) {
        if (xlinkProperties.isEnable()) {
            XxlJobLogger.log("pending");
            thirdPartySyncHandler.allSync();
            XxlJobLogger.log("ok");
        }
        return ReturnT.SUCCESS;
    }


    @XxlJob("xlinkSyncIncrData")
    @XxlJobTask(desc = "${xxl.xlink.syncIncrOrgData.desc:增量同步数据，5分钟执行一次}",
            cron = "${xxl.xlink.syncIncrOrgData.cron:0 0/5 * * * ? }",
            author = "pai-x-link")
    public ReturnT<String> syncIncrOrg(String param) {

        if (xlinkProperties.isEnable()) {
            XxlJobLogger.log("pending");
            thirdPartySyncHandler.incrSync();
            XxlJobLogger.log("ok");
        }
        return ReturnT.SUCCESS;
    }


    @XxlJob("cleanSyncLog")
    @XxlJobTask(desc = "${xxl.xlink.cleanSyncLog.desc:删除Log，每天凌晨执行}",
            cron = "${xxl.xlink.cleanSyncLog.cron:0 30 3 * * ? }",
            author = "pai-x-link")
    public ReturnT<String> cleanSyncLog(String param) {
        //删除上个月之前的日志
        Date lastMonth = DateUtil.offsetMonth(DateUtil.date(), -1);
        linkOrgSyncLogService.clean(lastMonth);
        return ReturnT.SUCCESS;
    }

}
