package gree;

import cn.flyrise.pai.xlink.gree.util.GreeHttpUtils;
import cn.hutool.core.text.StrFormatter;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.digest.MD5;
import cn.hutool.json.JSONUtil;
import cn.hutool.log.StaticLog;
import com.alibaba.fastjson.JSON;
import org.junit.Assert;
import org.junit.Test;

import java.util.HashMap;
import java.util.Map;

/**
 * 测试签名
 *
 * @author Y.zc
 */
@SuppressWarnings({"all"})
public class HttpSignTest {

    @Test
    public void greeTest(){
        String BODY_SIGN_FORMAT = "_appId={appId}&_appSecret={appSecret}&_time={appTime}&_body={appBody}";
        long curTime = System.currentTimeMillis();
        System.out.println(curTime);
        Map<String, String> signParams = new HashMap<>();
        signParams.put("appId", "xXbjBjBCmLu5E1QBYtQyN25RDfcQ");
        signParams.put("appSecret", "SPbOVzZIhUZJzbkNOPxpNGlIomtKXTyM");
        signParams.put("appTime", curTime + "");
        signParams.put("appBody", MD5.create().digestHex(StrUtil.removeAllLineBreaks(null)));
        String waitSignVal = StrUtil.format(BODY_SIGN_FORMAT, signParams, true);
        System.out.println(" ==src content: " + waitSignVal);
        System.out.println(" ==src sign: " + MD5.create().digestHex(waitSignVal));
        Assert.assertNotNull("请求失败：", waitSignVal);
    }

    @Test
    public void signTest() {
        String appId = "xXbjBjBCmLu5E1QBYtQyN25RDfcQ";
        String appSecret = "SPbOVzZIhUZJzbkNOPxpNGlIomtKXTyM";
        String serviceUrl = "http://psp-mdc-gw-app-sit.spaceplat.com";
        String url = serviceUrl + "/portal/v1/api/external/queryUserData";
        Request request = new Request();
        request.setPageNo(1);
        request.setPageSize(10);
//        request.setTime("2023-02-23");
        String requestStr = JSONUtil.toJsonStr(request);
        String responseBody = GreeHttpUtils.post(appId,appSecret,url,requestStr);
        Assert.assertNotNull("请求失败：", responseBody);
        //{"code":"200","msg":"请求成功","data":{"pageIndex":1,"totalPage":1,"pageSize":10,"totalCount":2,"rows":[{"realName":"资管系统","jobStatus":1,"gender":null,"phone":null,"accountAttributeList":[{"accountAttributeCode":"f31df173b37611ed988f0242ac1b0002","accountAttributeType":1,"accountType":10,"personCode":"f31d1852b37611ed988f0242ac1b0002"}],"userType":1,"id":null,"delFlag":0,"personCode":"f31d1852b37611ed988f0242ac1b0002","account":"ziguan","status":1},{"realName":"Kael","jobStatus":1,"gender":null,"phone":"15344377816","accountAttributeList":[{"accountAttributeCode":"84199bd80ad14cacb5c5f5d36f261f53","accountAttributeType":1,"accountType":10,"personCode":"b0766ca3308d4940909ae12f13f0827e"}],"userType":1,"id":null,"delFlag":0,"personCode":"b0766ca3308d4940909ae12f13f0827e","account":"15344377816","status":1}]}}
    }

    @Test
    public void getGreeOrg() {
        String appId = "xXbjBjBCmLu5E1QBYtQyN25RDfcQ";
        String appSecret = "SPbOVzZIhUZJzbkNOPxpNGlIomtKXTyM";
        String serviceUrl = "http://psp-mdc-gw-app-sit.spaceplat.com";
        String url = serviceUrl + "/portal/v1/api/external/queryOrgData";
        Request request = new Request();
        request.setPageNo(1);
        request.setPageSize(10);
//        request.setTime("2023-02-23");
        String requestStr = JSONUtil.toJsonStr(request);
        String responseBody = GreeHttpUtils.post(appId,appSecret,url,requestStr);
        Assert.assertNotNull("请求失败：", responseBody);
        //{"code":"200","msg":"请求成功","data":{"pageIndex":1,"totalPage":1,"pageSize":10,"totalCount":1,"rows":[{"orgName":"开放服务文档","parentOrgCode":"0","level":1,"orgCode":"ea42b0e363904e0989d6365bdee4fc8b","id":"1562","type":"goverOrg","delFlag":0}]}}

    }


    @Test
    public void signSpaceTest() {
        String appId = "xXbjBjBCmLu5E1QBYtQyN25RDfcQ";
        String appSecret = "SPbOVzZIhUZJzbkNOPxpNGlIomtKXTyM";
        String serviceUrl = "http://psp-mdc-gw-app-sit.spaceplat.com";
        String url = serviceUrl + "/space/v1/api/external/querySpaceResourceData";
        Request request = new Request();
        request.setPageNo(1);
        request.setPageSize(10);
//        request.setTime("2023-02-23");
        String requestStr = JSONUtil.toJsonStr(request);
        String responseBody = GreeHttpUtils.post(appId,appSecret,url,requestStr);
        StaticLog.info(JSON.parseObject(responseBody).getJSONObject("data").getJSONArray("rows").toJSONString());

        Assert.assertNotNull("请求失败：", responseBody);
        //{"code":"200","msg":"请求成功","data":{"pageIndex":1,"totalPage":1,"pageSize":10,"totalCount":2,"rows":[{"realName":"资管系统","jobStatus":1,"gender":null,"phone":null,"accountAttributeList":[{"accountAttributeCode":"f31df173b37611ed988f0242ac1b0002","accountAttributeType":1,"accountType":10,"personCode":"f31d1852b37611ed988f0242ac1b0002"}],"userType":1,"id":null,"delFlag":0,"personCode":"f31d1852b37611ed988f0242ac1b0002","account":"ziguan","status":1},{"realName":"Kael","jobStatus":1,"gender":null,"phone":"15344377816","accountAttributeList":[{"accountAttributeCode":"84199bd80ad14cacb5c5f5d36f261f53","accountAttributeType":1,"accountType":10,"personCode":"b0766ca3308d4940909ae12f13f0827e"}],"userType":1,"id":null,"delFlag":0,"personCode":"b0766ca3308d4940909ae12f13f0827e","account":"15344377816","status":1}]}}
    }


    @Test
    public void signLogoutTest() {
        String appId = "xXbjBjBCmLu5E1QBYtQyN25RDfcQ";
        String appSecret = "SPbOVzZIhUZJzbkNOPxpNGlIomtKXTyM";
        String serviceUrl = "http://psp-mdc-gw-app-sit.spaceplat.com";
        String url = serviceUrl + "/portal/v1/api/external/logout";
        String token = "ZH_00054:eyJ0eXBlIjoiSldUIiwiYWxnIjoiSFMyNTYifQ.eyJzdWIiOiJ0ZXN0IiwibG9naW5NZXRob2QiOm51bGwsInRlbmVtZW50Q29kZSI6IlpIXzAwMDU0IiwiZXhwIjoxNjc5MDI1MDU3LCJpYXQiOjE2Nzc3MjkwNTcsImFjY291bnQiOiJxaXllZnV3dSJ9.I629ppxYpcz1ZOAgnybzsAKXYSl1kZa6LPmbXEa-hLs";

        String responseBody = GreeHttpUtils.post(appId,appSecret,url,null,"Bearer "+token);
        Assert.assertNotNull("请求失败：", responseBody);
        //{"code":"200","msg":"请求成功","data":{"portalWebUrl":"https://portal-sso-sit.spaceplat.com/ubmp-workbench-view"}}

    }


    static class Request {

        private Integer pageNo;
        private Integer pageSize;
        private String time;

        public Integer getPageNo() {
            return pageNo;
        }

        public void setPageNo(Integer pageNo) {
            this.pageNo = pageNo;
        }

        public Integer getPageSize() {
            return pageSize;
        }

        public void setPageSize(Integer pageSize) {
            this.pageSize = pageSize;
        }

        public String getTime() {
            return time;
        }

        public void setTime(String time) {
            this.time = time;
        }
    }
}
